///--------------------------------------------------------------------
//  devctl.c
//
//  Copyright (c) 2005 National Semiconductor Corporation.
//  All Rights Reserved.
//
//  QNX6 Network Device Driver for National Semiconductor
//  DP83816 "MacPhyter" 10/100 Ethernet Controller.
//
//  The functions in this file handle devctl commands.
//
//--------------------------------------------------------------------


#include <macphy.h>

/*
	macphy_devctl

	Entry point from io-net to handle devctl commands.

	Parameters
	----------
		void *hdl: This driver's context structure.

		int dcmd: devctl command.

		void *data: data associated with the command.

		size_t size: size of the data buffer in octets.

		union _io_net_dcmd_ret_cred *ret: ?


	Returns
	-------
		EOK if okay, error code otherwise.
*/
int
macphy_devctl(
	void *hdl,
	int dcmd,
	void *data,
	size_t size,
	union _io_net_dcmd_ret_cred *ret )
{
	Nic_t				*nic = (Nic_t *)hdl;
	Nic_Macphy_Ext_t	*ext;
	int					status;

	status	= EOK;
	ext		= (Nic_Macphy_Ext_t *)nic->ext;

	switch( dcmd ) {
		case DCMD_IO_NET_NICINFO:
			memcpy( data, nic, min( size, sizeof( Nic_t ) ) );
			break;

        case DCMD_IO_NET_TXRX_COUNT:
            memcpy( data, &ext->txrx_count, 
                    min( size, sizeof( NicTxRxCount_t ) ) );
            break;

        case DCMD_IO_NET_PROMISCUOUS:
            /* do some work here for enabling/disabling promiscuous */
            status = ENOTSUP;
            break;

		default:
			status = ENOTSUP;
			break;
	}

	return( status );
}
