//--------------------------------------------------------------------
//  shutdown.c
//
//  Copyright (c) 2005 National Semiconductor Corporation.
//  All Rights Reserved.
//
//  QNX6 Network Device Driver for National Semiconductor
//  DP83816 "MacPhyter" 10/100 Ethernet Controller.
//
//  Functions to release resources on adapter shutdown.
//--------------------------------------------------------------------

#include <macphy.h>

/*******************************************************************************
 *
 */
int
macphy_shutdown1( int reg_hdl, void *hdl )
{
	Nic_t				*nic = (Nic_t *)hdl;
	Nic_Macphy_Ext_t	*ext;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;

	/* close down the Tx interface */
	ext->acceptingTx = 0;

	/* paranoia demands that we take the mutex */
    pthread_mutex_lock( &ext->mutex );

	if( ext->verbose )
	{
		slogf (_SLOGC_NETWORK, _SLOG_INFO, "devn-macphy: Shutdown1()");
	}

	/* if we had a phy, say goodbye  ................ */
	if (ext->mdi)
	{
		MDI_DisableMonitor(ext->mdi);
		MDI_DeRegister(&ext->mdi);
		ext->mdi = 0;
	}

    pthread_mutex_unlock( &ext->mutex );

	return (0);
}

/*******************************************************************************
 *
 */
int
macphy_shutdown2( int reg_hdl, void *hdl )
{
	Nic_t				*nic = (Nic_t *)hdl;
	Nic_Macphy_Ext_t	*ext;
	int					iobase;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;

    pthread_mutex_lock( &ext->mutex );

	if( ext->verbose ) {
		slogf (_SLOGC_NETWORK, _SLOG_INFO, "devn-macphy: Shutdown2()");
	}

	ext->acceptingTx = 0;

	pthread_cancel(ext->tid);
    pthread_join(ext->tid, NULL);

	/* Reset the MMU and wait for it to be un-busy. */

	/* Disable all interrupts. */

	InterruptDetach( ext->iid );
	ConnectDetach( ext->coid );
	ChannelDestroy( ext->chid );
	timer_delete( ext->timerid );

	/* tell io-net to finish off any packets sitting in the  tx queue */
	macphy_flush( ext->reg_hdl, nic );

	/* free the rx free list */
    pthread_mutex_unlock( &ext->mutex );

	/* destroy the mutexes */
	pthread_mutex_destroy( &ext->mutex );
	//munmap_device_io( ext->iobase, SMC_MMAP_SIZE );
	free( nic->ext );
	free( nic );

	return( 0 );
}
